<?php
/**
	@file campoListaDesplegable.php
	Fichero de la clase del campoListaDesplegable.
	@copy Copyright Ilke Benson 2008
	@author Miguel Jaque Barbero (mjaque@ilkebenson.com)
	@version 1.0
	@date 2008-02-19
**/

/**	Representa un campo de lista desplegable. 
	Está formado por un select.
**/
class CampoListaDesplegable extends Campo{
	protected $atributos = array('id','clase','lista','editable','tabindex','nulo');

	/**	Construye el campo
		Los atributos operativos son:
			- id: Identificador que se asignará al campo.
			- clase: Clase de CSS que se asignará al campo.
			- lista: Nombre de la ListaAuxiliar asociada al campo.
			- editable: Indica si la lista es inicialmente editable (sí/no).
			- nulo: Indica si la lista contendrá una opción nula (sí/no).
			- tabindex: Valor del índice de tabulación.
		@param $nodo Nodo campoLista de PAX (<pax:campoLista .../>).
	**/
	public function __construct($nodo){
		parent::__construct($nodo);
		if (!isset($this->id)) throw new ExcepcionCampoAtributoObligatorioNoDefinido('id',get_class($this));
	}
	/**	Devuelve el nodo que constituye el campo.
		@return DOMElement con el nodo que representa el campo
	**/
	public function verNodo(){
		$xml = new DOMDocument();
		// Creamos el nodo del campo de texto
		$select = $xml->createElement("select");
		$select->setAttribute("id", $this->id);
		if (isset($this->tabindex))
			$select->setAttribute("tabindex", $this->tabindex);
		if (isset($this->editable))
			if ($this->editable == 'no')
				$select->setAttribute("disabled", "disabled");
		if (isset($this->nulo))
			if ($this->nulo == 'sí'){
				$option = $xml->createElement('option');
				$option->setAttribute('value', null);
				$nombre = $xml->createTextNode('Sin definir');
				$option->appendChild($nombre);
				$select->appendChild($option);
			}
		
		// Cargamos los datos
		if (isset($this->lista)){
			$lista = new ListaAuxiliar($this->lista);
			$datos = $lista->ver();
			for($i=0; $i<sizeof($datos); $i++){
				$option = $xml->createElement("option");
				$option->setAttribute("value", $datos[$i]['id']);
				$nombre = $xml->createTextNode($datos[$i]['nombre']);
				$option->appendChild($nombre);
				$select->appendChild($option);
			}
		}

		return $select;
	}
}

///	Excepciones.

return true;	//Indicador de carga para __autoload
?>
