<?php
/**
	@file iu.php
	Fichero de la clase de interfaz de usuario.
	@copy Copyright Ilke Benson 2008
	@author Miguel Jaque Barbero (mjaque@ilkebenson.com)
	@version 1.0
	@date 2008-02-06
**/

/**	Clase abstracta de interfaz de usuario. El resto de clases de interfaz de usario derivan de ella. 
	Aporta métodos comunes para todos los interfaces gráficos y varios parámetros de configuración.
	La responsabilidad de las clases de interfaz de usuario es recibir las peticiones de los usuarios, extraer los parámetros del array $_REQUEST, comprobar que son válidos, llamar al controlador para obtener la respuesta y presentársela al usuario.
	También se encargan de la construcción de los interfaces gráficos (XHTML).
**/
abstract class Iu{

	//public $formatoFecha = 'Y-m-d';			//Formato internacional de fecha ISO.
	//public $formatoFechaHora = 'Y-m-d H:i:s';		//Formato internacional ISO.
	public $formatoFechaHora = 'd/m/Y H:i:s';		//Formato español.
	const NS_PAX = "http://www.ilkebenson.com/pax";
	const CABECERA_XHTML = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">';
	const CABECERA_HTTP_XHTML = "Content-Type: application/xhtml+xml; charset:utf-8; Cache-control: no-cache";
	const CABECERA_HTTP_AJAX = "Content-Type: application/xml; charset:utf-8; Cache-control: no-cache;";
	const DIR_JS = DIR_JS;
	const DIR_CSS = DIR_CSS;

	/**	Devuelve el código XHTML de las cabeceras requeridas para el campoFecha.
		@return El ćodigo XHTML de las cabeceras.
	**/
	public function cabecerasCampoFecha(){
		$xhtml = '<script type="text/JavaScript" src="iu/js/calendario/calendar.js"></script>'."\n";
		$xhtml .= '<script type="text/JavaScript" src="iu/js/calendario/lang/calendar-sp.js"></script>'."\n";
		$xhtml .= '<script type="text/JavaScript" src="iu/js/calendario/calendar-setup.js"></script>'."\n";
		$xhtml .= '<link href="iu/css/calendario/calendar-win2k-cold-1.css" rel="stylesheet" type="text/css"/>'."\n";

		return $xhtml;
	}	
	/**	Devuelve el código XHTML de un campo de fecha.
		El método cabecerasCampoFecha devuelve el código XHTML correspondiente a las cabeceras necesarias.
		@param $id Identificador que se asignará al campo. También se asignará un atributo name con ese valor.
		@param $clase Clase CSS que se asignará al campo.
		@param $valor Valor inicial del campo. Si no está definido, se pone la fecha actual.
		@return Devuelve código XHTML con el campo de fecha. Este campo incluye un calendario desplegable.
	**/
	protected function campoFecha($id,$clase,$valor){
		$xhtml = '<input type="text" size="10" maxlength="10"';
		if (isset($id)) $xhtml .= " id=\"$id\" name=\"$id\" ";
		if ($valor=='hoy') 
			$xhtml .= "value=\"".date($this->formatoFecha)."\" ";
		else 
			$xhtml .= "value=\"$valor\" ";

		$xhtml .= "/>";
		$xhtml .= '<script type="text/javascript">';
		$xhtml .= '	Calendar.setup({';
		$xhtml .= '	inputField : "'.$id.'",';
                //$xhtml .= '	ifFormat   : "%Y-%m-%d",';	//OJO, debería formarse utilizando $this->formatoFecha
                $xhtml .= '	ifFormat   : "%d/%m/%Y",';	//OJO, debería formarse utilizando $this->formatoFecha
                $xhtml .= ' 	button     : "'.$id.'"})';
		$xhtml .= '</script>';

		return $xhtml;
	}
	/**	Carga un fichero de interfaz en formato XML dentro de una plantilla XML.
		Para hacerlo, busca el elemento <pax:interfaz/> en la plantilla y lo sustituye por el documento XML del interfaz.
		También carga el script y el estilos propios del interfaz si así se indica.
		El script y el estilo propios corresponden a ficheros con el mismo nombre que el interfaz (nombre.js e nombre.css),
		se cargarán si el nodo página del interfaz tiene los atributos scriptPropio y estiloPropio con el valor "sí".
		@param $ficheroInterfaz Path del fichero XML del interfaz a cargar.
		@param &$plantilla Documento XML (DOMDocument) en el que se cargará el interfaz.
	**/
	protected function cargarInterfazEnPlantilla($ficheroInterfaz, &$plantilla){
		$docInterfaz = new DOMDocument();
		if (!$docInterfaz->load($ficheroInterfaz)) throw new ExcepcionIuCargarXML($ficheroInterfaz);
		$pagina = $docInterfaz->documentElement;
		$trozos = explode("/",$ficheroInterfaz);
		$trozos = explode(".", $trozos[sizeof($trozos)-1]);
		$nombre= $trozos[0];
		if ($pagina->hasAttribute("título")){
			$nodoTitulo = $plantilla->getElementsByTagName("title")->item(0);
			$nodoTitulo->childNodes->item(0)->nodeValue .= $pagina->getAttribute("título");
		}
		if ($pagina->hasAttribute("scriptPropio"))
			if ($pagina->getAttribute("scriptPropio") == "sí")	// Añadimos el script propio del interfaz
				$this->ponScript($plantilla, self::DIR_JS.$nombre.".js");
		if ($pagina->hasAttribute("estiloPropio"))
			if ($pagina->getAttribute("estiloPropio") == "sí")	// Añadimos el script propio del interfaz
				$this->ponCSS($plantilla, self::DIR_CSS.$nombre.".css");

		// "Reemplazamos" el nodo interfaz de la plantilla por el interfaz solicitado
		$nodoInterfaz = $plantilla->getElementsByTagNameNS(self::NS_PAX, "interfaz")->item(0);
		// El interfaz lo constituyen todos los elementos que son hijos del elemento página del fichero interfaz
		// Así que, primero los insertamos antes del nodo interfaz y luego eliminamos el nodo interfaz
		for ($i=0; $i<$pagina->childNodes->length; $i++){
			$nuevoNodo = $plantilla->importNode($pagina->childNodes->item($i),true);	// Es necesario importar el nodo al documento antes de hacer el reemplazo.
			$nodoInterfaz->parentNode->insertBefore($nuevoNodo, $nodoInterfaz);
		}
		$nodoInterfaz->parentNode->removeChild($nodoInterfaz);
	}
	/**	Carga la plantilla XHTML indicada
		@param $plantilla Path de la plantilla a cargar.
		@return Devuelve un documento XML (DOMDocument) con la plantilla cargada.
	**/
	protected function cargarPlantilla($plantilla){
		$docXML = new DOMDocument();
		if (!$docXML->load($plantilla)) throw new ExcepcionIuCargarXML($fichero);
		return $docXML;
		//$html = $plantilla->documentElement;
	}
	/**	Cierra elementos vacíos de un documento XML.
		Algunos elementos, como script y select no pueden ir vacíos en el XHTML porque FF e IE los presentan mal.
		@param $tag Nombre del elemento que debe ser cerrado.
		@param $xhtml Texto XML a procesar.
		@return Texto XML con los elementos vacíos corregidos.
	**/
	private function cerrarTag($tag, $xml){
		$indice = 0;
		while ($indice< strlen($xml)){
			$pos = strpos($xml, "<$tag ", $indice);
			if ($pos){
				$posCierre = strpos($xml, ">", $pos);
				if ($xml[$posCierre-1] == "/"){
					$xml = substr_replace($xml, "></$tag>", $posCierre-1, 2);
				}
				$indice = $posCierre;
			}
			else break;
		}
		return $xml;
	}
	/**	Genera la salida de un texto XHTML. Envía las cabeceras HTTP necesarias.
		@param $xhtml Texto del XHTML
	**/
	protected function enviarXHTML($xhtml){
		header(CABECERA_HTTP_XHTML);
		echo ($xhtml);
	}
	/**	Añade un fichero de hoja de estilo a la cabecera del documento
		@param &$xml Documento XML al que se añadirá el script.
		@param $path Path del fichero con la hoja de estilo.
	**/
	private function ponCSS(&$xml,$path){
		$head = $xml->getElementsByTagName("head")->item(0);
		$link = $xml->createElement("link");
		$link->setAttribute("rel", "stylesheet");
		$link->setAttribute("type", "text/css");
		$link->setAttribute("href", $path);	
		$head->appendChild($link);
	}
	/**	Añade un campo de Fecha al documento XML.
		El campo de fecha consiste en un campo de texto y un script.
		Además, hay que cargar en la página los scripts generales y el estilo del campo (si no están cargados).
		@param &$xml Referencia al documento XML al que añadir el campo.
		@param $nodo Nodo que debe sustituirse por el campo de Fecha.
	**/
	private function ponFecha(&$xml, &$nodo){
		// Creamos el nodo del campo de texto
		$campoTexto = $xml->createElement("input");
		$campoTexto->setAttribute("type", "text");
		$campoTexto->setAttribute("size", "10");
		$campoTexto->setAttribute("maxlength", "10");
		$id = $nodo->getAttribute("id");
		$campoTexto->setAttribute("id", $id);
		$campoTexto->setAttribute("name", $id);
		$fecha = date($this->formatoFecha);
		if ($nodo->hasAttribute("valor"))
			if ($nodo->getAttribute("valor") != "hoy")
				$fecha = $nodo->getAttribute("valor");
		$campoTexto->setAttribute("value", $fecha);

		// Creamos el nodo de script
		$script = $xml->createElement("script");
		$script->setAttribute("type", "text/JavaScript");
		$codigo = '	Calendar.setup({';
		$codigo .= '	inputField : "'.$id.'",';
                //codigo .= '	ifFormat   : "%Y-%m-%d",';	//OJO, debería formarse utilizando $this->formatoFecha
                $codigo .= '	ifFormat   : "%d/%m/%Y",';	//OJO, debería formarse utilizando $this->formatoFecha
                $codigo .= ' 	button     : "'.$id.'"})';	
		$script->appendChild($xml->createTextNode($codigo));

		// Creamos un span para unirlos
		$span = $xml->createElement("span");
		$span->appendChild($campoTexto);
		$span->appendChild($script);

		// Reemplazamos el nodo pax:fecha por el nodo de span
		$padre = $nodo->parentNode;
		$padre->replaceChild($span, $nodo);

		// Añadimos las cabeceras de script en el head del documento
		$this->ponScript($xml, self::DIR_JS.'calendario/calendar.js');
		$this->ponScript($xml, self::DIR_JS.'calendario/lang/calendar-sp.js');
		$this->ponScript($xml, self::DIR_JS.'calendario/calendar-setup.js');

		// Añadimos la hoja de estilo
		$this->ponCSS($xml, self::DIR_CSS.'calendario/calendar-win2k-cold-1.css');
	}
	/**	Añade una fichero de script a la cabecera del documento
		@param &$xml Documento XML al que se añadirá el script.
		@param $path Path del fichero con el script.
	**/
	private function ponScript(&$xml,$path){
		$head = $xml->getElementsByTagName("head")->item(0);
		$script = $xml->createElement("script");
		$script->setAttribute("type", "text/JavaScript");
		$script->setAttribute("src", $path);	
		//$codigo = $xml->createCDATASection(";");		// Es necesaria una sección de código para que se ponga el tag de cierre y no haya error sintáctico.
		//$script->appendChild($codigo);
		$head->appendChild($script);
	}
	/**	Procesa un documento XML para susituir los nodos propios de un espacio de nombres.
		Al procesar cada nodo, se sustituye por el nodo de la clase correspondiente.
		El nodo sustitutivo se obtiene invocando el método verNodo de la clase creada.
		@param &$xml Referencia al Documento XML
		@param $ns Espacio de nombres referido. Si no se indica se utilizará el de PAX.
	**/
	protected function procesar(&$xml, $ns){
		if (!isset($ns)) $ns = self::NS_PAX;
		$nodos = $xml->getElementsByTagNameNS($ns, "*");
		//Siempre quitamos el primer elemento, porque la lista... ¡se reorganiza sola!
		while ($nodos->length > 0){
			$nodo = $nodos->item(0);
			//echo $nodo->nodeName." - ";
			if ($nodo->nodeName == "pax:página") continue;	//Este nodo no se procesa
			$trozos = explode(":",$nodo->nodeName);
			$clase = ucfirst($trozos[1]);
			$campo = new ${'clase'}($nodo);
			
			// Reemplazamos el nodo por el de la clase
			$nuevoNodo = $xml->importNode($campo->verNodo(), true);
			$padre = $nodo->parentNode;
			$padre->replaceChild($nuevoNodo, $nodo);
		}
	}
	/**	Devuelve un objeto en una respuesta AJAX.
	 	Transforma el objeto en un mensaje XML y lo envía al cliente.
		@param $objeto El objeto respuesta.
	**/
	public function responderPorAjax($objeto){
		$respuesta = new Xml($objeto);
		header(self::CABECERA_HTTP_AJAX);
		echo $respuesta->verXML();
	}
	/**	Sustituye el primer nodo de un elemento XML, identificado por su 'id', por un nodo de texto.
		Se utiliza para sustituir contenidos de <p>, <h1>, etc.
		@param &$dom Documento DOM en el que se operará.
		@param $id Valor del atributo 'id' que identifica al nodo del $dom sobre el que se operará.
		@param $texto Texto que contendrá el nodo de texto
	**/
	protected function sustituirTexto(&$dom, $id, $texto){
		$xp = new DomXPath($dom);
		$res = $xp->query("//*[@id = '$id']");
		$nodo = $res->item(0);
		if ($nodo->hasChildNodes())
			$nodo->removeChild($nodo->firstChild);
		$nodoTexto = $dom->createTextNode($texto);
		$nodo->appendChild($nodoTexto);
	}
	/**	Genera el XHTML correspondiente a un documento (generalmente una plantilla con interfaz).
		@param $doc Documento XML (DOMDocument) con la plantilla.
		@return Texto XHTML correspondiente al documento.
	**/
	protected function verXHTML($doc){
		$xhtml = self::CABECERA_XHTML."\n";
		$xhtml .= $doc->saveXML($doc->documentElement);
		// El problema es que saveXML no cierra correctamente los tags de script, los hace vacíos y eso a FF y a IE no les gusta.
		// La opción de utilizar saveHTML hace que los atributos disabled y checked tampoco se generen conforme a XHTML 1.0
		// Así que la única opción que queda es procesar la salida para corregir el error.
		$xhtml = $this->cerrarTag("script", $xhtml);
		$xhtml = $this->cerrarTag("select", $xhtml);
		return $xhtml;
	}
}

// Excepciones.
class ExcepcionIuCargarXML extends Excepcion{
	public function __construct($fichero){
		$titulo = "El fichero XML '$fichero' no pudo ser cargado";
		$texto = "La aplicación no ha podido cargar un fichero del tipo XML.";
		$solucionProgramador = "Compruebe que el fichero existe, que hay permisos para leerlo y que su sintaxis XML es correcta.";
		parent::__construct($titulo,$texto,$solucionProgramador);
	}
}
class ExcepcionIuInsertarInterfaz extends Excepcion{
	public function __construct($fichero){
		$titulo = "El interfaz correspondiente a '$fichero' no pudo insertarse en la plantilla";
		$texto = "La aplicación no ha podido sustituir el nodo de la plantilla con el nodo del interfaz.";
		$solucionProgramador = "Compruebe que la plantilla contiene un nodo de interfaz.";
		parent::__construct($titulo,$texto,$solucionProgramador);
	}
}

return true;	//Indicador de carga para __autoload
?>
