<?php
/**
	@file mysql.php
	Fichero con la clase MySQL.
	Copyright Ilke Benson 2008
	@author Miguel Jaque Barbero (mjaque@ilkebenson.com)
	@version 1.0
	@date 2008-03-15
**/

/**	Base de Datos MySQL.
**/
class Mysql{
	//Atributos
	public $mysqli;

	/** 	Crea el objeto de base de datos.
		No utilizamos el constructor __construct para permitir que sus subclases sean singletons.
		@param $host Host del servidor de base de datos.
		@param $bd Nombre de la base de datos a la que se conectará.
		@param $usuario Nombre del usuario de conexión.
		@param $clave Clave del usuario de conexión.
	**/
	public function Mysql($host, $bd, $usuario, $clave){
		$this->conectar($host, $bd, $usuario, $clave);
	}
	/** 	Crea la conexión con la base de datos.
		@param $host Host del servidor de base de datos.
		@param $bd Nombre de la base de datos a la que se conectará.
		@param $usuario Nombre del usuario de conexión.
		@param $clave Clave del usuario de conexión.
	**/
	public function conectar($host, $bd, $usuario, $clave){
		$this->host = $host;
		$this->usuario = $usuario;
		$this->clave = $clave;
		$this->bd = $bd;
		$this->mysqli = new mysqli($host, $usuario, $clave, $bd);
		if (!$this->mysqli) throw new ExcepcionMySQLConexion($host,$usuario);

		return this;
	}
	/**	Actualiza datos en una tabla MySQL con clave simple (un campo).
		@param $tabla Nombre de la tabla en la que se realizará la inserción.
		@param $nombreClave Nombre de la columna clave por la que se actualizará la tabla.
		@param $valorClave Valor de la clave en la tupla/s que se actualizarán.
		@param $valores Array asociativo con los nombres de las columnas y los valores a insertar.
		@return Devuelve el número de tuplas afectadas por la operación.
	**/
	public function actualizar($tabla, $nombreClave, $valorClave, $valores){
		$consulta = "UPDATE $tabla SET ";
		foreach($valores as $columna => $valor){
			if ($valor == NULL) $consulta .= "$columna = NULL,";
			else $consulta .= "$columna = '$valor',";
		}
		$consulta = substr($consulta, 0, strlen($consulta)-1);
		if ($valorClave == NULL)
			$consulta .= " WHERE $nombreClave = NULL";
		else
			$consulta .= " WHERE $nombreClave = '$valorClave'";

		$this->ejecutar($consulta);
		
		return $this->mysqli->affected_rows;
	}
	/**	Actualiza el valor de un atributo en una tabla, aplicando un filtro de selección opcional.
		@param $tabla Nombre de la tabla sobre la que se realizará la actualización.
		@param $atributo Nombre del atributo que se actualizará.
		@param $valor Nuevo valor para el atributo.
		@param $filtro (opcional) Sentencia SQL de filtro.
		@return Devuelve el número de tuplas afectadas por la operación.
	**/
	public function actualizarAtributo($tabla, $atributo, $valor, $filtro){
		$consulta = "UPDATE $tabla ";
		if ($valor == NULL)
			$consulta .= "SET $atributo = NULL ";
		else
			$consulta .= "SET $atributo = '$valor' ";
		if (isset($filtro))
			$consulta .= "WHERE $filtro";
		$this->ejecutar($consulta);

		return $this->mysqli->affected_rows;
	}
	/**	Actualiza datos en una tabla MySQL con clave simple (un campo).
		@param $tabla Nombre de la tabla en la que se realizará la inserción.
		@param $claveCompuesta Array asociativo por los nombres de las columnas que forman la clave y con los valores de cada una.
		@param $valores Array asociativo con los nombres de las columnas y los valores a insertar.
		@return Devuelve el número de tuplas afectadas por la operación.
	**/
	public function actualizarPorClaveCompuesta($tabla, $claveCompuesta, $valores){
		$consulta = "UPDATE $tabla SET ";
		foreach ($valores as $columna => $valor)
			if ($valor == NULL) $consulta .= "$columna = NULL,";
			else $consulta .= "$columna = '$valor',";
		$consulta = substr($consulta, 0, strlen($consulta)-1);
		$consulta .= " WHERE ";
		foreach ($claveCompuesta as $columna => $valor){
			if ($valor == NULL) $consulta .= "$columna = NULL AND ";
			else $consulta .= "$columna = '$valor' AND ";
		}
		$consulta = substr($consulta, 0, strlen($consulta)-strlen(' AND '));
		$this->ejecutar($consulta);
		
		return $this->mysqli->affected_rows;
	}
	/**	Borra entradas de una tabla según el filtro indicado.
	 	@param $tabla Nombre de la tabla en la que se realizará el borrado.
		@param $filtro Array asociativo de los nombres de columnas y valores que constituirán el filtro de borrado.
	**/
	public function borrar($tabla, $filtro){
		$consulta = "DELETE FROM $tabla WHERE ";
		foreach($filtro as $columna => $valor){
			if ($valor == NULL) $consulta .= "$columna = NULL AND ";
			else $consulta .= "$columna = '$valor' AND ";
		}
		$consulta = substr($consulta, 0, strlen($consulta)-5);
		$this->ejecutar($consulta);
	}
	/**	Carga los datos y la estructura de una tabla de la base de datos de un fichero de texto.
		@param $tabla Nombre de la tabla.
		@param $fichero Path del fichero.
	**/
	public function cargarTabla($tabla, $fichero){
		$comando = 'mysql -u '.$this->usuario.' --password='.$this->clave.' '.$this->bd.' < '.$fichero;
		exec($comando, $salida, $ret);
		//TODO: Control de errores. ¿Cómo es la salida?
		if ($ret != 0) throw new ExcepcionMySQLErrorCargandoTabla($tabla, $fichero);
	}
	/**	Codifica un resultado en UTF-8.
		Esta función se utiliza cuando la base de datos está codificada en ISO.
		@param &$resultado Array de arrays asociativos con los resultados a codificar.
		@param $campos Array con los nombres de los campos a codificar en UTF-8.
		@return Devuelve el mismo array con los campos codificados.
	**/
	public function codificarResultado($resultado, $campos){
		$maxResultado = sizeof($resultado);
		$maxCampos = sizeof($campos);
		for($i=0; $i<$maxResultado;$i++)
			for($j=0;$j<$maxCampos;$j++)
				$resultado[$i][$campos[$j]] = utf8_encode($resultado[$i][$campos[$j]]);
		return $resultado;
	}
	/**	Ejecuta una sentencia SQL sin devolver datos (para inserción, actualización y borrado).
		@param $consulta Sentencia SQL de la consulta.
	**/
	public function ejecutar($consulta){
		$this->mysqli->query('SET NAMES utf8');
		$this->mysqli->query($consulta);
		if ($this->mysqli->errno){
			throw new ExcepcionMySQLError($consulta,$this->mysqli->error);
		}
	}
	/**	Escapa los caracteres de un array de parámetros para evitar ataques tipo SQL Injection.
		Debe utilizarse para todos los parámetros antes de llamar a funciones de la clase Mysql.
		@param $parametros Array con los parámetros a escapar.
		@return Array de parámetros escapados.
	**/
	public function escaparCaracteres($parametros){
	//TODO: Ver como aplicar esta función.
		$max = sizeof($parametros);
		for($i=0;$i<$max;$i++)
			$parametros[$i] = mysql_real_escape_string($parametros[$i]);
		return $parametros;
	}
	/**	Guarda los datos y la estructura de una tabla de la base de datos en un fichero de texto.
		La tabla puede cargarse de nuevo con cargarTabla.
		@param $tabla Nombre de la tabla.
		@param $fichero Path del fichero en el que se guardará la tabla.
	**/
	public function guardarTabla($tabla, $fichero){
		$comando = 'mysqldump --add-drop-table -u '.$this->usuario.' --password='.$this->clave.' '.$this->bd.' '.$tabla.' > '.$fichero;
		exec($comando, $salida, $ret);
		if ($ret != 0) throw new ExcepcionMySQLErrorGuardandoTabla($tabla, $fichero);
	}
	/**	Inserta datos en una tabla MySQL.
		@param $tabla Nombre de la tabla en la que se realizará la inserción.
		@param $valores Array asociativo con los nombres de las columnas y los valores a insertar.
		@return Devuelve el identificador del la línea insertada si la tabla tiene un campo autoincrementado
	**/
	public function insertar($tabla, $valores){
		$consulta = "INSERT INTO $tabla SET ";
		foreach($valores as $columna => $valor){
			if ($valor === NULL) $consulta .= "$columna = NULL,";
			else $consulta .= "$columna = '$valor',";
		}
		$consulta = substr($consulta, 0, strlen($consulta)-1);
		$this->ejecutar($consulta);
		
		return $this->mysqli->insert_id;
	}
	/**	Ejecuta una consulta de SELECT contra la Base de Datos.
	 	@param $consulta La consulta a ejecutar.
		@return Devuelve un array con todas las filas de datos.
	**/
	public function seleccionar($consulta){
		$this->mysqli->query('SET NAMES utf8');
		$respuesta = array();
		$resultado = $this->mysqli->query($consulta);
		if ($this->mysqli->errno) throw new ExcepcionMySQLError($consulta,$this->mysqli->error);

		while ($tupla = $resultado->fetch_array())
			array_push($respuesta, $tupla);

		$resultado->free();
		
		return $respuesta;
	}
	/**	Devuelve las tuplas con los atributos de una tabla, aplicando el filtro.
		@param $tabla Nombre de la tabla.
		@param $atributos Array de atributos a seleccionar.
		@param $filtro Sentencia SQL de filtro.
		@param $limite Número máximo de resultados. Si el límite es false, no habrá límite.
		@return Devuelve un array con los datos del resultado.
	**/ 
	public function seleccionarAtributosPorFiltro($tabla, $atributos, $filtro, $limite=1){
		$listaAtributos = "";
		foreach($atributos as $atributo)
			$listaAtributos .= $atributo.",";
		$listaAtributos = substr($listaAtributos, 0, strlen($listaAtributos) -1);
		$consulta = "SELECT $listaAtributos FROM $tabla WHERE $filtro";
		$respuesta = $this->seleccionar($consulta);
		if ($limite)
			$consulta .= " LIMIT $limite";

		return $respuesta[0];
	}
	/**	Devuelve las tuplas de una tabla con el atributo al valor indicado.
		@param $tabla Nombre de la tabla.
		@param $atributo Nombre del atributo para el filtro de selección.
		@param $valor Valor del atributo.
		@param $limite Número máximo de resultados. Si el límite es false, no habrá límite.
		@param $ordenacion Claúsula SQL para la ordenación de resultados.
		@return Devuelve un array con los datos del resultado. Se seleccionan todas las columnas de la tabla.
	**/ 
	public function seleccionarPorAtributo($tabla, $atributo, $valor, $limite=1, $ordenacion=false){
		if ($valor == NULL)
			$consulta = "SELECT * FROM $tabla WHERE $atributo = NULL";
		else
			$consulta = "SELECT * FROM $tabla WHERE $atributo = '$valor'";
		if ($limite)
			$consulta .= " LIMIT $limite";
		if  ($ordenacion)
			$consulta .=" ORDER BY $ordenacion";
		
		$respuesta = $this->seleccionar($consulta);

		return $respuesta;
	}
	/**	Devuelve las tuplas de una tabla, sin filtro.
		@param $tabla Nombre de la tabla.
		@param $limite Número máximo de resultados. Si el límite es false, no habrá límite.
		@return Devuelve un array con los datos del resultado. Se seleccionan todas las columnas de la tabla.
	**/ 
	public function seleccionarTabla($tabla, $limite=1000){

		$consulta = "SELECT * FROM $tabla ";
		$respuesta = $this->seleccionar($consulta);
		if ($limite)
			$consulta .= " LIMIT $limite";

		return $respuesta;
	}
	/**	Devuelve una fecha en el formato de MySQL
		@param $fechaEsp Texto con la fecha en formato español (DD/MM/AAAA o DD/MM/AA)
		@param $hora Booleano indicando si la fecha se mostrará con hora. Por defecto, se mostrará con hora si la fecha original la tiene.
		@return Texto con la fecha en formato de MySQL (AAAA-MM-DD)
	**/
	public function verFechaMysql($fechaEsp, $hora){
	Registro::anotar($fechaEsp);
		if ($fechaEsp == null) return null;
		if (strlen($fechaEsp) < 6 ) throw new ExcepcionMySQLFormatoFechaInvalido($fechaEsp);
		if (strlen($fechaEsp) > 19) throw new ExcepcionMySQLFormatoFechaInvalido($fechaEsp);
		$trozos = explode(" ",$fechaEsp);
		$formatoMysql = "%Y-%m-%d";

		//Procesamos la fecha
		$fecha = $trozos[0];
		$trozosFecha = explode("/",$fecha);
		if (sizeof($trozosFecha) != 3) throw new ExcepcionMySQLFormatoFechaInvalido($fechaEsp);
		$dia = $trozosFecha[0];
		$mes = $trozosFecha[1];
		$ano = $trozosFecha[2];
		if ((strlen($ano) == 4) AND (($ano > '2037') OR ($ano < '1901'))) throw new ExcepcionMySQLFechaFueraRango($fechaEsp);

		//Procesamos la hora
		$horas = 0;
		$minutos = 0;
		$segundos = 0;
		if ($hora) $formatoMysql .= " %H:%M:%S";
		if ((sizeof($trozos) > 1) AND ($hora !== false)){
			$formatoMysql .= " %H:%M:%S";
			$hora = $trozos[1];
			$trozosHora = explode(":",$hora);
			if (sizeof($trozosHora) != 3) throw new ExcepcionMySQLFormatoFechaInvalido($fechaEsp);
			$horas = $trozosHora[0];
			$minutos = $trozosHora[1];
			$segundos = $trozosHora[2];
		}

		//Construimos la fecha UNIX
		$tiempo = mktime($horas, $minutos, $segundos, $mes, $dia, $ano);

		//La pasamos a formato MySQL
		return strftime($formatoMysql,$tiempo);
	}
	/**	Transforma una fecha en formato Mysql a formato español (DD/MM/AAAA).
		@param $fechaMysql Texto con la fecha en formato Mysql (YYYY-MM-DD HH:MM:SS).
		@return Texto con la fecha en formato español.
	**/
	public function verFechaEsp($fechaMysql){
		if ($fechaMysql == null) return null;

		$trozos = explode(' ',$fechaMysql);
		$fecha = $trozos[0];
		$hora = $trozos[1];

		$trozosFecha = explode('-',$fecha);
		$anio = $trozosFecha[0];
		$mes = $trozosFecha[1];
		$dia = $trozosFecha[2];

		return $dia.'/'.$mes.'/'.$anio;
	}
}

// 	Excepciones
class ExcepcionMySQLConexion extends Excepcion{
	public function __construct($host, $usuario){
		$titulo = "No se pudo conectar con el servidor de MySQL";
		$texto = "La aplicación no ha podido realizar la conexión al servidor de base de datos MySQL en $host con el usuario $usuario.";
		$solucion = "Revise los datos de conexión (host, usuario y clave) y compruebe que el servidor MySQL está en funcionamiento y es accesible.";
		parent::__construct($titulo,$texto,$solucion);
	}
}
class ExcepcionMySQLSeleccionBd extends Excepcion{
	public function __construct($bd){
		$titulo = "No se pudo seleccionar la base de datos $bd en MySQL";
		$texto = "La aplicación no ha podido seleccionar la base de datos $bd para realizar las operaciones.";
		$solucion = "Compruebe el nombre de la base de datos en los parámetros de configuración y que exista en el servidor MySQL indicado.";
		parent::__construct($titulo,$texto,$solucion);
	}
}
class ExcepcionMySQLError extends Excepcion{
	public function __construct($consulta, $error){
		$titulo = "Ha fallado la operación de base de datos.";
		$texto = "La aplicación no ha podido realizar la operación de base de datos solicitada.";
		$solucion = "La consulta que produjo el error es ($consulta) y el error producido es ($error).";
		parent::__construct($titulo,$texto,$solucion);
	}
}
class ExcepcionMySQLFormatoFechaInvalido extends Excepcion{
	public function __construct($fecha){
		$titulo = "El formato de fecha no es válido.";
		$texto = "La aplicación no ha podido utilizar una fecha recibida ($fecha) porque el formato es inválido.";
		$solucion = "Los formatos de fecha soportados son: DD/MM/AA, DD/MM/AAAA, DD/M/AAAA, DD/M/AA con o sin horas con el formato HH:MM:SS. Por ejemplo '13/02/1968 17:45:32'";
		parent::__construct($titulo,$texto,$solucion);
	}
}
class ExcepcionMySQLFechaFueraRango extends Excepcion{
	public function __construct($fecha){
		$titulo = "La fecha está fuera de rango.";
		$texto = "La aplicación no puede transformar la fecha ($fecha) porque excede los rangos permitidos.";
		$solucionProgramador = "La función Mysql::verFechaMysql utiliza la función mktime, que tiene limitado su rango de validez. Tendrás que programar otra función.";
		$solucionUsuario = "Revise el año de la fecha introducida. Si realmente necesita una fecha con ese año, póngase en contacto con el administrador.";
		parent::__construct($titulo,$texto,$solucionProgramador, $solucionUsuario);
	}
}
class ExcepcionMySQLErrorCargandoTabla extends Excepcion{
	public function __construct($tabla, $fichero){
		$titulo = "Error al cargar la tabla '$tabla' del fichero '$fichero'.";
		$texto = "La aplicación no ha podido cargar la tabla del fichero indicado.";
		$solucionProgramador = "Revise los parámetros de la llamada y los permisos del fichero.";
		parent::__construct($titulo,$texto,$solucionProgramador);
	}
}
class ExcepcionMySQLErrorGuardandoTabla extends Excepcion{
	public function __construct($tabla, $fichero){
		$titulo = "Error al guardar la tabla '$tabla' en el fichero '$fichero'.";
		$texto = "La aplicación no ha podido guardar la tabla en el fichero indicado.";
		$solucionProgramador = "Revise los parámetros y los permisos.";
		parent::__construct($titulo,$texto,$solucionProgramador);
	}
}
return true;	//Indicador de carga para __autoload
?>
