<?php
/**
	@file parametro.php
	Fichero con la clase Parametro.
	Copyright Ilke Benson 2008
	@author Miguel Jaque Barbero (mjaque@ilkebenson.com)
	@version 1.0
	@date 2008-04-27

**/
/**	Implementa un parámetro de $_REQUEST
**/
class Parametro{

	//Atributos
	public $valor;		//Valor del Parámetro
	public $nombre;		//Nombre del Parámetro

	/**	Lee el valor del parámetro.
		Se consideran nulos las siguientes situaciones:
			- No se ha recibido el parámetro.
			- El parámetro tiene como valor 'undefined'.
			- El parámetro no tiene valor.
		@param $nombre Nombre del parámetro. Debe ser el nombre en el array $_REQUEST.
		@param $vacio Booleano que indica si el Parámetro puede ser vacío.
		@param $nulo Booleano que indica si el Parámetro puede ser nulo (no definido).
	**/
	public function __construct($nombre, $nulo, $vacio){

		if ((!isset($nombre)) OR ($nombre=='')) throw new ExcepcionParametroNombreInvalido();
		$this->nombre = $nombre;
		if (!$nulo){
			if (!isset($_REQUEST[$nombre])) throw (new ExcepcionParametroInexistente($nombre));
			if ($_REQUEST[$nombre] == 'undefined') throw (new ExcepcionParametroIndefinido($nombre));
			if ($_REQUEST[$nombre] == '') throw (new ExcepcionParametroIndefinido($nombre));
		}
		if (!$vacio)
			if ($_REQUEST[$nombre] == "") throw (new ExcepcionParametroVacio($nombre));

		if (!isset($_REQUEST[$nombre]))  $this->valor = null;
		elseif ($_REQUEST[$nombre] == 'undefined') $this->valor = null;
		elseif ($_REQUEST[$nombre] == '') $this->valor = null;
		else $this->valor = addslashes($_REQUEST[$nombre]);
	}
}

//Excepciones
class ExcepcionParametroNombreInvalido extends Excepcion{
	public function __construct(){
		$titulo = "No se ha especificado ningún nombre para el parámetro";
		$texto = "La aplicación no ha podido leer el parámetro solicitado porque no se ha indicado ningún nombre para él.";
		$solucionProgramador = "Revise los parámetros pasados al constructor de la clase.";
		parent::__construct($titulo,$texto,$solucionProgramador);
	}
}
class ExcepcionParametroInexistente extends Excepcion{
	public function __construct($parametro){
		$titulo = "Falta el parámetro requerido '$parametro'.";
		$texto = "La aplicación no ha recibido el parámetro '$parametro', que es imprescindible para realizar la operación.";
		$solucion = "Compruebe que ha completado todos los campos obligatorios del formulario y reintente la operación.";
		parent::__construct($titulo,$texto,$solucion);
	}
}
class ExcepcionParametroIndefinido extends Excepcion{
	public function __construct($parametro){
		$titulo = "El Parámetro '$parametro' No está Definido.";
		$texto = "La aplicación ha recibido el valor 'undefined' para el parámetro '$parametro', por lo que es imposible realizar la operación.";
		$solucionProgramador = "Compruebe que ha asignado valores a los campos del formulario y que se asignan valores por defecto.";
		$solucionUsuario = "Compruebe que ha completado todos los campos obligatorios del formulario y reintente la operación.";
		parent::__construct($titulo,$texto,$solucion, $solucionUsuario);
	}
}
class ExcepcionParametroVacio extends Excepcion{
	public function __construct($parametro){
		$titulo = "El parámetro '$parametro' tiene un valor vacío";
		$texto = "La aplicación ha recibido el parámetro '$parametro' con un valor vacío. Este parámetro requiere un valor válido.";
		$solucionProgramador = "Compruebe que todos los parámetros requeridos son enviados y que los nombres están correctamente escritos.";
		$solucionUsuario = "Compruebe que ha completado todos los campos obligatorios del formulario y reintente la operación.";
		parent::__construct($titulo,$texto,$solucionProgramador,$solucionUsuario);
	}
}

return true;	//Indicador de carga para __autoload
?>
